/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workactivity.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workactivity.entity.Activity;
import com.jeeplus.modules.workactivity.service.ActivityService;

/**
 * 工作流附加类Controller
 * @author 杨帆
 * @version 2018-01-24
 */
@Controller
@RequestMapping(value = "${adminPath}/workactivity/activity")
public class ActivityController extends BaseController {

	@Autowired
	private ActivityService activityService;
	
	@ModelAttribute
	public Activity get(@RequestParam(required=false) String id) {
		Activity entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = activityService.get(id);
		}
		if (entity == null){
			entity = new Activity();
		}
		return entity;
	}
	
	/**
	 * 工作流附加类列表页面
	 */
	@RequiresPermissions("workactivity:activity:list")
	@RequestMapping(value = {"list", ""})
	public String list(Activity activity, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Activity> page = activityService.findPage(new Page<Activity>(request, response), activity); 
		model.addAttribute("page", page);
		return "modules/workactivity/activityList";
	}

	/**
	 * 查看，增加，编辑工作流附加类表单页面
	 */
	@RequiresPermissions(value={"workactivity:activity:view","workactivity:activity:add","workactivity:activity:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Activity activity, Model model) {
		model.addAttribute("activity", activity);
		return "modules/workactivity/activityForm";
	}

	/**
	 * 保存工作流附加类
	 */
	@RequiresPermissions(value={"workactivity:activity:add","workactivity:activity:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Activity activity, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, activity)){
			return form(activity, model);
		}
		if(!activity.getIsNewRecord()){//编辑表单保存
			Activity t = activityService.get(activity.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(activity, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			activityService.save(t);//保存
		}else{//新增表单保存
			activityService.save(activity);//保存
		}
		addMessage(redirectAttributes, "保存工作流附加类成功");
		return "redirect:"+Global.getAdminPath()+"/workactivity/activity/?repage";
	}
	
	/**
	 * 删除工作流附加类
	 */
	@RequiresPermissions("workactivity:activity:del")
	@RequestMapping(value = "delete")
	public String delete(Activity activity, RedirectAttributes redirectAttributes) {
		activityService.delete(activity);
		addMessage(redirectAttributes, "删除工作流附加类成功");
		return "redirect:"+Global.getAdminPath()+"/workactivity/activity/?repage";
	}
	
	/**
	 * 批量删除工作流附加类
	 */
	@RequiresPermissions("workactivity:activity:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			activityService.delete(activityService.get(id));
		}
		addMessage(redirectAttributes, "删除工作流附加类成功");
		return "redirect:"+Global.getAdminPath()+"/workactivity/activity/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workactivity:activity:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Activity activity, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工作流附加类"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Activity> page = activityService.findPage(new Page<Activity>(request, response, -1), activity);
    		new ExportExcel("工作流附加类", Activity.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出工作流附加类记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workactivity/activity/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workactivity:activity:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Activity> list = ei.getDataList(Activity.class);
			for (Activity activity : list){
				try{
					activityService.save(activity);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条工作流附加类记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条工作流附加类记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入工作流附加类失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workactivity/activity/?repage";
    }
	
	/**
	 * 下载导入工作流附加类数据模板
	 */
	@RequiresPermissions("workactivity:activity:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工作流附加类数据导入模板.xlsx";
    		List<Activity> list = Lists.newArrayList(); 
    		new ExportExcel("工作流附加类数据", Activity.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workactivity/activity/?repage";
    }
	
	
	

}